=== modified file 'src/BamfLauncherIcon.cpp'
--- src/BamfLauncherIcon.cpp	2010-11-08 19:49:33 +0000
+++ src/BamfLauncherIcon.cpp	2010-11-09 18:20:51 +0000
@@ -27,6 +27,7 @@
     
     g_free (icon_name);
     
+    g_signal_connect (app, "urgent-changed", (GCallback) &BamfLauncherIcon::OnUrgentChanged, this);
     g_signal_connect (app, "running-changed", (GCallback) &BamfLauncherIcon::OnRunningChanged, this);
     g_signal_connect (app, "active-changed", (GCallback) &BamfLauncherIcon::OnActiveChanged, this);
     g_signal_connect (app, "user-visible-changed", (GCallback) &BamfLauncherIcon::OnUserVisibleChanged, this);
@@ -132,3 +133,10 @@
     BamfLauncherIcon *self = (BamfLauncherIcon *) data;
     self->SetActive (active);
 }
+
+void
+BamfLauncherIcon::OnUrgentChanged (BamfView *view, gboolean urgent, gpointer data)
+{
+    BamfLauncherIcon *self = (BamfLauncherIcon *) data;
+    self->SetUrgent (urgent);
+}

=== modified file 'src/BamfLauncherIcon.h'
--- src/BamfLauncherIcon.h	2010-11-03 11:44:21 +0000
+++ src/BamfLauncherIcon.h	2010-11-09 18:20:51 +0000
@@ -30,6 +30,7 @@
     static void OnUserVisibleChanged (BamfView *view, gboolean visible, gpointer data);
     static void OnActiveChanged (BamfView *view, gboolean active, gpointer data);
     static void OnRunningChanged (BamfView *view, gboolean running, gpointer data);
+    static void OnUrgentChanged (BamfView *view, gboolean urgent, gpointer data);
 };
 
 #endif // BAMFLAUNCHERICON_H

=== modified file 'src/Launcher.cpp'
--- src/Launcher.cpp	2010-11-09 04:44:13 +0000
+++ src/Launcher.cpp	2010-11-09 18:20:51 +0000
@@ -290,6 +290,30 @@
         _anim_handle = g_timeout_add (1000 / 60 - 1, &Launcher::AnimationTimeout, this);
 }
 
+bool Launcher::IconNeedsAnimation (LauncherIcon *icon, struct timeval current)
+{
+    struct timeval enter_time = icon->ShowTime ();
+    if (TimeDelta (&current, &enter_time) < ANIM_DURATION_SHORT)
+        return true;
+    
+    struct timeval hide_time = icon->HideTime ();
+    if (TimeDelta (&current, &hide_time) < ANIM_DURATION_SHORT)
+        return true;
+    
+    struct timeval running_time = icon->RunningTime ();
+    if (TimeDelta (&current, &running_time) < ANIM_DURATION_SHORT)
+        return true;
+    
+    if (icon->Urgent ())
+    {
+        struct timeval urgent_time = icon->UrgentTime ();
+        if (TimeDelta (&current, &urgent_time) < (ANIM_DURATION_LONG * 4))
+            return true;
+    }
+    
+    return false;
+}
+
 bool Launcher::AnimationInProgress ()
 {
     // performance here can be improved by caching the longer remaining animation found and short circuiting to that each time
@@ -317,19 +341,8 @@
     // animations happening on specific icons
     LauncherModel::iterator it;
     for (it = _model->begin  (); it != _model->end (); it++)
-    {
-        struct timeval enter_time = (*it)->ShowTime ();
-        if (TimeDelta (&current, &enter_time) < ANIM_DURATION_SHORT)
-            return true;
-        
-        struct timeval hide_time = (*it)->HideTime ();
-        if (TimeDelta (&current, &hide_time) < ANIM_DURATION_SHORT)
-            return true;
-        
-        struct timeval running_time = (*it)->RunningTime ();
-        if (TimeDelta (&current, &running_time) < ANIM_DURATION_SHORT)
-            return true;
-    }
+        if (IconNeedsAnimation (*it, current))
+            return true;
     
     return false;
 }
@@ -477,6 +490,15 @@
         if (icon->Running ())
         {
           arg.backlight_intensity = running_progress * BACKLIGHT_STRENGTH;
+          
+          if (icon->Urgent ())
+          {
+              struct timeval urgent_time = icon->UrgentTime ();
+              int urgent_ms = TimeDelta (&current, &urgent_time);
+              double urgent_progress = (double) MIN (1.0f, (float) urgent_ms / (float) (ANIM_DURATION_LONG * 4));
+              
+              arg.backlight_intensity *= 0.5f + (float) (std::cos (M_PI * 4.0f * urgent_progress)) * 0.5f;
+          }
         }
         else
         {

=== modified file 'src/Launcher.h'
--- src/Launcher.h	2010-11-09 04:44:13 +0000
+++ src/Launcher.h	2010-11-09 18:20:51 +0000
@@ -79,24 +79,25 @@
   void OnTriggerMouseEnter (int x, int y, unsigned long button_flags, unsigned long key_flags);
   void OnTriggerMouseLeave (int x, int y, unsigned long button_flags, unsigned long key_flags);
   
+  bool IconNeedsAnimation  (LauncherIcon *icon, struct timeval current);
   bool AnimationInProgress ();
-  void SetTimeStruct (struct timeval *timer, struct timeval *sister = 0, int sister_relation = 0);
+  void SetTimeStruct       (struct timeval *timer, struct timeval *sister = 0, int sister_relation = 0);
   
-  void EnsureAnimation ();
+  void EnsureAnimation    ();
   void SetupAutohideTimer ();
   
-  float DnDExitProgress ();
+  float DnDExitProgress  ();
   float GetHoverProgress ();
   float AutohideProgress ();
 
   void SetHover   ();
   void UnsetHover ();
-  void SetHidden (bool hidden);
+  void SetHidden  (bool hidden);
   
   std::list<RenderArg> RenderArgs (nux::Geometry &box_geo);
 
-  void OnIconAdded (void *icon_pointer);
-  void OnIconRemoved (void *icon_pointer);
+  void OnIconAdded    (void *icon_pointer);
+  void OnIconRemoved  (void *icon_pointer);
   void OnOrderChanged ();
 
   void OnIconNeedsRedraw (void *icon);

=== modified file 'src/LauncherIcon.cpp'
--- src/LauncherIcon.cpp	2010-11-08 19:49:33 +0000
+++ src/LauncherIcon.cpp	2010-11-09 18:20:51 +0000
@@ -24,14 +24,17 @@
   _show_time.tv_sec = 0;
   _hide_time.tv_sec = 0;
   _running_time.tv_sec = 0;
+  _urgent_time.tv_sec = 0;
 
   _show_time.tv_usec = 0;
   _hide_time.tv_usec = 0;
   _running_time.tv_usec = 0;
+  _urgent_time.tv_usec = 0;
 
   _active  = false;
   _running = false;
   _visible = false;
+  _urgent  = false;
 
   _background_color = nux::Color::White;
   _mouse_inside = false;
@@ -201,6 +204,11 @@
   return _running_time;
 }
 
+struct timeval LauncherIcon::UrgentTime ()
+{
+  return _urgent_time;
+}
+
 void
 LauncherIcon::SetVisible (bool visible)
 {
@@ -243,6 +251,18 @@
   needs_redraw.emit (this);
 }
 
+void LauncherIcon::SetUrgent (bool urgent)
+{
+  if (urgent == _urgent)
+    return;
+  
+  _urgent = urgent;
+  
+  if (urgent)
+      gettimeofday (&_urgent_time, NULL);
+  
+  needs_redraw.emit (this);
+}
 
 void LauncherIcon::Remove ()
 {
@@ -291,3 +311,9 @@
 {
   return _running;
 }
+
+bool
+LauncherIcon::Urgent ()
+{
+  return _urgent;
+}

=== modified file 'src/LauncherIcon.h'
--- src/LauncherIcon.h	2010-11-08 19:49:33 +0000
+++ src/LauncherIcon.h	2010-11-09 18:20:51 +0000
@@ -41,6 +41,7 @@
     bool Visible ();
     bool Active  ();
     bool Running ();
+    bool Urgent  ();
 
     void RecvMouseEnter ();
 
@@ -53,6 +54,7 @@
     struct timeval ShowTime ();
     struct timeval HideTime ();
     struct timeval RunningTime ();
+    struct timeval UrgentTime ();
     
     LauncherIconType Type ();
     
@@ -73,8 +75,9 @@
 protected:
 
     void SetVisible (bool visible);
-    void SetActive (bool active);
+    void SetActive  (bool active);
     void SetRunning (bool running);
+    void SetUrgent  (bool urgent);
     void Remove ();
     
     void SetIconType (LauncherIconType type);
@@ -107,11 +110,13 @@
     bool             _visible;
     bool             _active;
     bool             _running;
+    bool             _urgent;
     int              _sort_priority;
     LauncherIconType _icon_type;
     struct timeval   _show_time;
     struct timeval   _hide_time;
     struct timeval   _running_time;
+    struct timeval   _urgent_time;
 };
 
 #endif // LAUNCHERICON_H

